<?php
/*------------------------------------------------------------------------------
 PropertyReader.php 2020-10-28
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/

namespace Gambio\Shop\Properties\Database\Readers;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;
use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\Properties\Database\Criterias\CheckStockBeforeShoppingCartCriteria;
use Gambio\Shop\Properties\Database\Readers\Interfaces\PropertyReaderInterface;
use Gambio\Shop\Properties\ProductModifiers\Database\ValueObjects\PropertyModifierIdentifier;
use Gambio\Shop\Properties\Properties\Builders\CombinationBuilderInterface;
use Gambio\Shop\Properties\Properties\Collections\CombinationCollection;
use Gambio\Shop\Properties\Properties\Collections\CombinationCollectionInterface;
use Gambio\Shop\Properties\Properties\Entities\Combination;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationEan;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationId;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationModel;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationOrder;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationQuantity;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationSurcharge;
use Gambio\Shop\Properties\Properties\ValueObjects\CombinationWeight;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Vpe;

/**
 * Class PropertyReader
 *
 * @package Gambio\Shop\Properties\Database\Readers
 */
class PropertyReader implements PropertyReaderInterface
{
    
    /**
     * @var array
     */
    protected $cache = [
        'getCombinationModifierIds' => [],
        'hasProperties'             => []
    ];
    
    /**
     * @var Connection
     */
    private $connection;
    /**
     * @var CombinationBuilderInterface
     */
    private $builder;
    /**
     * @var CheckStockBeforeShoppingCartCriteria
     */
    private $stockCriteria;
    
    
    /**
     * PropertyReader constructor.
     *
     * @param Connection                           $connection
     * @param CombinationBuilderInterface          $builder
     * @param CheckStockBeforeShoppingCartCriteria $criteria
     */
    public function __construct(
        Connection $connection,
        CombinationBuilderInterface $builder,
        CheckStockBeforeShoppingCartCriteria $criteria
    ) {
        $this->connection    = $connection;
        $this->builder       = $builder;
        $this->stockCriteria = $criteria;
    }
    
    
    /**
     * @inheritDoc
     */
    public function getCombinationModifierIds(int $combinationId): iterable
    {
        
        
        $result  = [];
        $records = $this->connection->createQueryBuilder()
            ->select('distinct products_id, properties_values_id')
            ->from('products_properties_index')
            ->where('products_properties_index.products_properties_combis_id = :combination_id')
            ->setParameter('combination_id', $combinationId)
            ->execute()
            ->fetchAll();
        foreach ($records as $row) {
            if (empty($result[(int)$row['products_id']])) {
                $result[(int)$row['products_id']] = [];
            }
            $result[(int)$row['products_id']][] = (int)$row['properties_values_id'];
        }
        
        return $result;
    }
    
    
    /**
     * @param int    $productId
     * @param string $whereAnd
     * @param string $having
     *
     * @return string
     */
    protected function createQuery(int $productId, string $whereAnd = '', string $having = ''): string
    {
        return "
            SELECT `ppc`.`products_properties_combis_id`,
                   `ppc`.`products_id`,
                   `ppc`.`sort_order`,
                   `ppc`.`combi_model`,
                   `ppc`.`combi_weight`,
                   `ppc`.`combi_ean`,
                   `ppc`.`combi_quantity`,
                   `ppc`.`combi_shipping_status_id`,
                   `ppc`.`combi_price_type`,
                   `ppc`.`combi_price`,
                   `ppc`.`products_vpe_id`,
                   `ppc`.`vpe_value`,
                   `pvpe`.`products_vpe_name`,
                   `p`.`gm_show_weight`,
                   `p`.`use_properties_combis_weight`,
                   `p`.`use_properties_combis_quantity`,
                   `p`.`use_properties_combis_shipping_time`,
                   `p`.`products_quantity`
            FROM `products` AS `p`
            INNER JOIN `products_properties_combis` AS `ppc`
                ON ppc.products_id = p.`products_id`
            INNER JOIN `products_properties_combis_values` AS `ppcvx2`
                ON `ppc`.`products_properties_combis_id` = `ppcvx2`.`products_properties_combis_id`
            LEFT JOIN `products_vpe` as `pvpe`
             ON `ppc`.`products_vpe_id` = `pvpe`.`products_vpe_id`
             AND `pvpe`.`language_id` = {$this->activeLanguageId()}
             WHERE p.`products_id` = {$productId}
            {$whereAnd}
            GROUP BY
                `ppc`.`products_properties_combis_id`,
                   `ppc`.`products_id`,
                   `ppc`.`sort_order`,
                   `ppc`.`combi_model`,
                   `ppc`.`combi_weight`,
                   `ppc`.`combi_ean`,
                   `ppc`.`combi_quantity`,
                   `ppc`.`combi_shipping_status_id`,
                   `ppc`.`combi_price_type`,
                   `ppc`.`combi_price`,
                   `ppc`.`products_vpe_id`,
                   `ppc`.`vpe_value`,
                   `pvpe`.`products_vpe_name`,
                   `ppc`.`vpe_value`,
                   `p`.`gm_show_weight`,
                   `p`.`products_quantity`,
                   `p`.`use_properties_combis_weight`,
                   `p`.`use_properties_combis_quantity`,
                   `p`.`use_properties_combis_shipping_time`
            {$having}
            ";
    }
    
    
    /**
     * @inheritDoc
     */
    public function getCombinationsFor(SellingUnitId $id): CombinationCollectionInterface
    {
        $result         = new CombinationCollection();
        
        [$whereAnd, $having] = $this->filterProperties($id);
        $sql  = $this->createQuery($id->productId()->value(), $whereAnd, $having);
        $data = $this->connection->query($sql)->fetchAll();
        
        $nonLinearSurcharge = (count($data) > 0) ? $this->hasNonLinearSurcharge($id->productId()->value()) : false;
        
        foreach ($data as $row) {
            $result[] = $this->createCombination($row, $nonLinearSurcharge);
        }
        
        return $result;
    }
    
    
    /**
     * @param ProductId $productId
     *
     * @return int
     */
    protected function getNumberOfProperties(ProductId $productId): int
    {
        
        $records = $this->connection->createQueryBuilder()
            ->select('count(distinct pv.properties_id) AS total')
            ->from('products_properties_combis', 'ppc')
            ->leftJoin('ppc',
                       'products_properties_combis_values',
                       'ppcvx2',
                       'ppc.products_properties_combis_id = ppcvx2.products_properties_combis_id')
            ->leftJoin('ppcvx2',
                       'properties_values',
                       'pv',
                       'pv.properties_values_id = ppcvx2.properties_values_id')
            ->where('ppc.products_id = :products_id')
            ->setParameter('products_id', $productId->value())
            ->setMaxResults(1)
            ->execute()
            ->fetchAll();
        
        foreach ($records as $row) {
            return (int)$row['total'];
        }
        
        return 0;
    }
    
    
    /**
     * @inheritDoc
     */
    public function hasProperties(ProductId $productId): bool
    {
        return $this->getNumberOfProperties($productId) > 0;
    }
    
    
    /**
     * @param string $type
     * @param float  $surcharge
     * @param bool   $nonLinear
     *
     * @return CombinationSurcharge|null
     */
    protected function createSurcharge(string $type, float $surcharge, bool $nonLinear): ?CombinationSurcharge
    {
        if (($surcharge && $type === 'calc') || $nonLinear) {
            return new CombinationSurcharge($surcharge, $nonLinear);
        }
        
        return null;
    }
    
    
    /**
     * @param int    $id
     * @param string $name
     * @param float  $value
     *
     * @return Vpe|null
     */
    protected function createVpe(int $id, string $name, float $value): ?Vpe
    {
        if ($id && $name && $value) {
            return new Vpe($id, $name, $value);
        }
        
        return null;
    }
    
    
    /**
     * @param int       $productId
     * @param bool|null $hasSurcharge
     *
     * @return bool
     */
    protected function hasNonLinearSurcharge(int $productId): bool
    {
        $result = $this->hasSurcharge($productId);
        if ($result) {
            $sql = "SELECT
                    round((p.products_price + ppc.combi_price) / ppc.vpe_value,2) AS base_price
                FROM
                    products AS p
                LEFT JOIN
                    products_properties_combis AS ppc ON (ppc.products_id = p.products_id)
                WHERE
                    p.products_id = {$productId}
                    AND ppc.vpe_value != 0
                GROUP BY
                    base_price";
            try {
                $data      = $this->connection->query($sql)->fetchAll();
                $countData = count($data);
                $result    = $countData > 1 || $countData === 0;
            } catch (DBALException $e) {
                echo $e->getMessage();
            }
        }
        
        return $result;
    }
    
    
    /**
     * @param int $productId
     *
     * @return bool
     * @throws
     */
    protected function hasSurcharge(int $productId): bool
    {
        
        $sql = "SELECT
                    combi_price
                FROM
                    products_properties_combis
                WHERE
                    products_id = {$productId}
                GROUP BY
                    combi_price";
        try {
            return count($this->connection->query($sql)->fetchAll()) > 1;
        } catch (DBALException $e) {
            return true;
        }
    }
    
    
    /**
     * @return int
     */
    protected function activeLanguageId(): int
    {
        return (int)$_SESSION['languages_id'];
    }
    
    
    /**
     * @inheritDoc
     */
    public function getCombinationFor(SellingUnitId $id): ?Combination
    {
        $list = [];
        foreach ($id->modifiers() as $modifier) {
            if ($modifier instanceof PropertyModifierIdentifier) {
                $list[] = $modifier;
            }
        }
        if (count($list) == $this->getNumberOfProperties($id->productId())) {
            $result = $this->getCombinationsFor($id);
            
            return $result[0];
        }
        
        return null;
    }
    
    
    /**
     * @param array $row
     * @param bool  $nonLinearSurcharge
     *
     * @param bool  $cheapest
     *
     * @return Combination
     */
    protected function createCombination(array $row, bool $nonLinearSurcharge, bool $cheapest = false): Combination
    {
        $this->builder->withId(new CombinationId($row['products_properties_combis_id']))
            ->withOrder(new CombinationOrder((int)$row['sort_order']))
            ->withModel(new CombinationModel($row['combi_model']))
            ->withEan(new CombinationEan($row['combi_ean']))
            ->withQuantity(new CombinationQuantity($row['combi_quantity']))
            ->withSurcharge($this->createSurcharge($row['combi_price_type'],
                                                   (float)$row['combi_price'],
                                                   $nonLinearSurcharge))
            ->withVpe($this->createVpe((int)$row['products_vpe_id'],
                                       (string)$row['products_vpe_name'],
                                       (float)$row['vpe_value']));
        
        if ((int)$row['use_properties_combis_weight']) {
            $show = (int)$row['gm_show_weight'] > 0;
            $this->builder->withWeight(new CombinationWeight($row['combi_weight'], $show));
        }
        if($cheapest)
            return $this->builder->buildCheapest();
        else
            return $this->builder->build();
    }
    
    
    /**
     * @param int $productId
     *
     * @return int
     * @throws DBALException
     */
    protected function getCombinationStockConfiguration(int $productId): int
    {
        $sql  = "SELECT `use_properties_combis_quantity` FROM `products` WHERE `products_id` = $productId";
        $data = $this->connection->query($sql)->fetchAll();
        
        return (int)$data[0]['use_properties_combis_quantity'];
    }
    
    
    /**
     * @param SellingUnitId $id
     *
     * @return mixed|void
     * @throws DBALException
     */
    public function getCheapestCombinationFor(SellingUnitId $id): ?Combination
    {
        
        [$propertyFilter, $having] = $this->filterProperties($id);
        [$stockFilter, $orderBy, $limit] = $this->filterStock($id);
        
        $sql  = $this->createQuery(
            $id->productId()->value(),
            $propertyFilter.$stockFilter,
            $having.' ORDER BY coalesce(`ppc`.`combi_price`,0) '.$limit
        );

        $data = $this->connection->query($sql)->fetchAll();
        
        $nonLinearSurcharge = (count($data) > 0) ? $this->hasNonLinearSurcharge($id->productId()->value()) : false;
        
        foreach ($data as $row) {
            return $this->createCombination($row, $nonLinearSurcharge, count($data) > 1);
        }
        
        return null;
    }
    
    
    /**
     * @param SellingUnitId $id
     *
     * @return string[]
     */
    protected function filterProperties(SellingUnitId $id)
    {
        $whereAnd = '';
        $having = '';
        $propertyIdList = [];
        foreach ($id->modifiers() as $modifierId) {
            if ($modifierId instanceof PropertyModifierIdentifier) {
                $propertyIdList[] = $modifierId->value();
            }
        }
        $listCount = count($propertyIdList);
        if ($listCount) {
            $inList   = implode(',', $propertyIdList);
            $whereAnd = "\n AND ppcvx2.properties_values_id IN ($inList)";
            $having   = "\n HAVING COUNT(ppcvx2.products_properties_combis_values_id) = $listCount";
        }
        return [$whereAnd, $having];
    }
    
    
    /**
     * @param SellingUnitId $id
     *
     * @return string[]
     * @throws DBALException
     */
    protected function filterStock(SellingUnitId $id)
    {
        $stockFilter   = '';
        $orderBy     = '';

        $configuration  = $this->getCombinationStockConfiguration($id->productId()->value());
        if ($this->stockCriteria->checkStockForCombinationConfiguration($configuration)) {
            $stockField = $this->stockCriteria->getStockSourceConfiguration(
                $configuration,
                '`p`.`products_quantity`',
                '`ppc`.`combi_quantity`'
            );
            $stockFilter   = "\n AND coalesce($stockField,0) > 0";
            $orderBy     = "\n ORDER BY coalesce($stockField,0)";
        }
        
        return [$stockFilter, $orderBy, "\n limit 2"];
    
    }
}