<?php
/*------------------------------------------------------------------------------
 PropertyQuantityReadService.php 2020-10-27
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\Properties\Database\Services;

use Exception;
use Gambio\Shop\Properties\Database\Criterias\CheckStockBeforeShoppingCartCriteria;
use Gambio\Shop\Properties\Database\Exceptions\ProductDoesntUsePropertyQuantityException;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertyQuantityReadServiceInterface;
use Gambio\Shop\Properties\Properties\ValueObjects\AbstractPropertyQuantity;
use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollection;
use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollectionInterface;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertiesReaderServiceInterface;
use Gambio\Shop\Properties\ProductModifiers\Database\ValueObjects\PropertyModifierIdentifier;
use Gambio\Shop\Properties\Properties\Collections\CombinationCollectionInterface;
use Gambio\Shop\Properties\Properties\Entities\Combination;
use Gambio\Shop\Properties\Properties\ValueObjects\AvailableCombinationQuantity;
use Gambio\Shop\Properties\Properties\ValueObjects\PropertyTotalAvailableQuantity;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Interfaces\QuantityInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Interfaces\ScopedQuantityInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;
use ProductDataInterface;

/**
 * Class PropertyQuantityReadService
 * @package Gambio\Shop\Properties\Database\Services
 */
class PropertyQuantityReadService implements PropertyQuantityReadServiceInterface
{
    /**
     * @var bool
     */
    protected $attributeStockCheck;
    
    /**
     * @var PropertiesReaderServiceInterface
     */
    protected $propertiesReaderService;
    
    /**
     * @var bool
     */
    protected $stockCheck;
    
    /**
     * @var CheckStockBeforeShoppingCartCriteria
     */
    private $criteria;
    
    
    /**
     * PropertyQuantityReadService constructor.
     *
     * @param PropertiesReaderServiceInterface     $propertiesReaderService
     * @param CheckStockBeforeShoppingCartCriteria $criteria
     * @param bool                                 $stockCheck
     * @param bool                                 $attributeStockCheck
     */
    public function __construct(
        PropertiesReaderServiceInterface $propertiesReaderService,
        CheckStockBeforeShoppingCartCriteria $criteria,
        bool $stockCheck,
        bool $attributeStockCheck
    ) {
        $this->propertiesReaderService = $propertiesReaderService;
        $this->criteria                = $criteria;
        $this->stockCheck              = $stockCheck;
        $this->attributeStockCheck     = $attributeStockCheck;
    }
    
    
    /**
     * @inheritDoc
     */
    public function getAvailableQuantityBy(
        SellingUnitId $id,
        ProductDataInterface $product,
        QuantityInterface $requested
    ): ?AbstractPropertyQuantity {
        try {
            if ($this->criteria->getStockSourceConfiguration($product->getPropertiesQuantityCheck(), false, true)) {
                
                $combinations = $this->propertiesReaderService->getCombinationsFor($id);
                
                if (count($combinations) === 1) {
                    return $this->getAvailableQuantityByCombination($id, $combinations[0], $product, $requested);
                }
                
                if (!empty($combinations)) {
                    return $this->sumUpAvailableQuantities($id, $product, $combinations, $requested);
                }
            }
        } catch (Exception $e) {
            //silent the exception
        }
        
        return null;
    }
    
    
    /**
     * @param SellingUnitId        $sellingUnitId
     * @param Combination          $combination
     * @param ProductDataInterface $product
     * @param QuantityInterface    $requested
     *
     * @return AvailableCombinationQuantity|null
     */
    
    public function getAvailableQuantityByCombination(
        SellingUnitId $sellingUnitId,
        Combination $combination,
        ProductDataInterface $product,
        QuantityInterface $requested
    ): ?AvailableCombinationQuantity {
        try {
            
            $collection = $this->createModifierIdCollection($sellingUnitId);
            
            $qty = $combination->quantity()->value();
            
            if ($requested && $requested instanceof ScopedQuantityInterface) {
                /** @var ScopedQuantityInterface $requested */
                $qty -= $requested->scope()->quantityFor($sellingUnitId->productId(), $collection);
            }
            
            return new AvailableCombinationQuantity($qty, $product->measureUnit(), $combination->id(),
                $this->stockCheck, $this->attributeStockCheck, $product->getPropertiesQuantityCheck(), $collection);
        } catch (ProductDoesntUsePropertyQuantityException $e) {
            return null;
        }
    }
    
    
    /**
     * @param SellingUnitId                  $sellingUnitId
     * @param ProductDataInterface           $product
     * @param CombinationCollectionInterface $combinations
     * @param QuantityInterface|null         $requested
     *
     * @return PropertyTotalAvailableQuantity|null
     */
    public function sumUpAvailableQuantities(
        SellingUnitId $sellingUnitId,
        ProductDataInterface $product,
        CombinationCollectionInterface $combinations,
        ?QuantityInterface $requested = null
    ): ?PropertyTotalAvailableQuantity {
        if (count($combinations) === 0) {
            return null;
        }
        
        try {
            $qty = 0;
            foreach ($combinations as $combination) {
                $qty += $combination->quantity()->value();
            }
            $collection = $this->createModifierIdCollection($sellingUnitId);
            
            if ($requested && $requested instanceof ScopedQuantityInterface) {
                /** @var ScopedQuantityInterface $requested */
                $qty -= $requested->scope()->quantityFor($sellingUnitId->productId(), $collection);
            }
            
            return new PropertyTotalAvailableQuantity($qty, $product->measureUnit(), $this->stockCheck,
                $this->attributeStockCheck, $product->getPropertiesQuantityCheck(), $collection);
        } catch (ProductDoesntUsePropertyQuantityException $e) {
            return null;
        }
    }
    
    
    /**
     * @param SellingUnitId $id
     *
     * @return ModifierIdentifierCollectionInterface
     */
    protected function createModifierIdCollection(SellingUnitId $id): ModifierIdentifierCollectionInterface
    {
        $propertyIdList = [];
        foreach ($id->modifiers() as $modifierId) {
            if ($modifierId instanceof PropertyModifierIdentifier) {
                $propertyIdList[] = $modifierId;
            }
        }
        
        return new ModifierIdentifierCollection($propertyIdList);
    }
}
