<?php
/*------------------------------------------------------------------------------
 PropertyModifierReader.php 2020-11-06
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/
declare(strict_types=1);

namespace Gambio\Shop\Properties\ProductModifiers\Database\Readers;

use CI_DB_query_builder;
use Gambio\Shop\Language\ValueObjects\LanguageId;
use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\ProductModifiers\Database\Core\DTO\Modifiers\ModifierDTOBuilderInterface;
use Gambio\Shop\ProductModifiers\Database\Core\DTO\Modifiers\ModifierDTOCollection;
use Gambio\Shop\ProductModifiers\Database\Core\DTO\Modifiers\ModifierDTOCollectionInterface;
use Gambio\Shop\ProductModifiers\Database\Core\Readers\Interfaces\ModifierReaderCompositeInterface;
use Gambio\Shop\Properties\ProductModifiers\Database\ValueObjects\PropertyGroupIdentifier;
use Gambio\Shop\Properties\ProductModifiers\Database\ValueObjects\PropertyModifierIdentifier;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;

/**
 * Class PropertyModifierReader
 *
 * @package Gambio\Shop\Properties\ProductModifiers\Database\Readers
 * @codeCoverageIgnore
 * @internal currently untestable due to the usage of the CI_DB_query_builder
 * documentation about stock rules can be found at:
            https://sources.gambio-server.net/gambio/gxdev/-/wikis/Stock-management
 */
class PropertyModifierReader implements ModifierReaderCompositeInterface
{

    /**
     * @var ModifierDTOBuilderInterface
     */
    protected $builder;
    /**
     * @var CI_DB_query_builder
     */
    protected $queryBuilder;
    /**
     * @var bool
     */
    protected $stockCheck;
    /**
     * @var bool
     */
    protected $attributeStockCheck;
    /**
     * @var bool
     */
    protected $stockAllowCheckout;
    /**
     * @var bool
     */
    protected $checkStockBeforeShoppingCart;


    /**
     * PropertyModifierReader constructor.
     *
     * @param CI_DB_query_builder $queryBuilder
     * @param ModifierDTOBuilderInterface $builder
     * @param bool $stockCheck
     * @param bool $attributeStockCheck
     * @param bool $stockAllowCheckout
     * @param bool                        $checkStockBeforeShoppingCart
     */
    public function __construct(
        CI_DB_query_builder $queryBuilder,
        ModifierDTOBuilderInterface $builder,
        bool $stockCheck,
        bool $attributeStockCheck,
        bool $stockAllowCheckout,
        bool $checkStockBeforeShoppingCart
    ) {
        $this->queryBuilder        = $queryBuilder;
        $this->builder             = $builder;
        $this->stockCheck          = $stockCheck;
        $this->attributeStockCheck = $attributeStockCheck;
        $this->stockAllowCheckout  = $stockAllowCheckout;
        $this->checkStockBeforeShoppingCart = $checkStockBeforeShoppingCart;
    }


    /**
     * @inheritDoc
     */
    public function getModifierByProduct(ProductId $id, LanguageId $languageId): ModifierDTOCollectionInterface
    {
        $result = new ModifierDTOCollection();

        $sql = "SELECT p.properties_id, p.display_type, pv.display_image, pvd.values_name, pvd.properties_values_id, pv.`value_price`
                    FROM products_properties_combis ppc
                        INNER JOIN products_properties_combis_values ppcv
                            ON ppc.products_properties_combis_id = ppcv.products_properties_combis_id
                        INNER JOIN properties_values_description pvd
                            ON pvd.properties_values_id = ppcv.properties_values_id AND pvd.language_id = {$languageId->value()}
                        INNER JOIN properties_values pv
                            ON pv.properties_values_id = ppcv.properties_values_id
                        INNER JOIN properties p
                            ON p.properties_id = pv.properties_id
                    WHERE ppc.products_id = {$id->value()}
                    GROUP BY p.properties_id, pvd.values_name, pvd.properties_values_id
                    ORDER BY ppc.sort_order";

        $data = $this->queryBuilder->query($sql)->result_array();

        foreach ($data as $item) {

            [$pricePrefix, $priceValueFloat] = $this->pricePrefixAndValueFromFloat((float)$item['value_price']);

            $result->addModifier($this->builder->withId(new PropertyModifierIdentifier((int)$item['properties_values_id']))
                ->withGroupId(new PropertyGroupIdentifier((int)$item['properties_id']))
                ->withType($item['display_type'])
                ->withName($item['values_name'])
                ->withPrice($priceValueFloat)
                ->withPricePrefix($pricePrefix)
                ->withImage($item['display_image'] ? 'product_images/property_images/' . $item['display_image'] : '')
                ->withSource('property')
                ->build());
        }

        return $result;
    }


    /**
     * @param float $priceValueFloat
     *
     * @return mixed[]
     */
    protected function pricePrefixAndValueFromFloat(float $priceValueFloat): array
    {
        $pricePrefix     = $priceValueFloat >= 0 ? '+ ' : '- ';
        $priceValueFloat = $pricePrefix === '- ' ? $priceValueFloat * -1 : $priceValueFloat;

        return [$pricePrefix, $priceValueFloat];
    }
    
    
    /**
     * @param SellingUnitId $id
     *
     * @return array
     * @throws DBALException
     */
    protected function getProductPropertiesConfiguration(SellingUnitId $id) {
        $selectedPropertyValues = $this->selectedPropertyValues($id);
        $indexedSelected = [];
        $result = [
            'selected_values' => [],
            'use_properties_combis_quantity' => '',
            'properties_dropdown_mode' => '',
            'properties' => []
        ];
        
        $sql ="
                select `products`.`use_properties_combis_quantity`
                        , `products`.`properties_dropdown_mode`
                        , `p`.`properties_id`
                        , `p`.`display_type`
                        , `pv`.`properties_values_id`
                        , `pv`.`display_image`
                        , `pv`.`value_price`
                        , `pvd`.`values_name`
                from `products`
                     inner join (
                            select distinct `products_id`, `properties_values_id`
                            from `products_properties_combis` `ppc`
                            inner join `products_properties_combis_values` `ppcv`
                                on ppc.`products_properties_combis_id` = ppcv.`products_properties_combis_id`
                            where ppc.products_id = {$id->productId()->value()}
                     ) `ppas`
                         on `ppas`.`products_id` = `products`.`products_id`
                     inner join `properties_values` `pv`
                         on `pv`.`properties_values_id` = `ppas`.`properties_values_id`
                     inner join `properties_values_description` `pvd`
                         on `pvd`.`properties_values_id` = `pv`.`properties_values_id`
                        and `pvd`.`language_id` = {$id->language()->value()}
                     inner join `properties` `p`
                         on `p`.`properties_id` = `pv`.`properties_id`
                where `products`.`products_id` = {$id->productId()->value()}
                order by `p`.`sort_order`, pv.`sort_order`";
        $data = $this->queryBuilder->query($sql)->result_array();
        foreach ($data as $record) {
            $result['use_properties_combis_quantity'] = (int)$record['use_properties_combis_quantity'];
            $result['properties_dropdown_mode'] = $record['properties_dropdown_mode'];
            if(!isset($result['properties'][$record['properties_id']])) {
                $result['properties'][$record['properties_id']] = [
                    'values'       => [],
                    'display_type' => $record['display_type'],
                    'selected'     => false
                ];
            }
            $result['properties'][$record['properties_id']]['values'][(int)$record['properties_values_id']] = [
                'display_image' => $record['display_image'],
                'value_price'   => (double)$record['value_price'],
                'values_name'   => $record['values_name'],
                'selectable'   => $result['properties_dropdown_mode'] == '',
                'selected'     => false
            ];

            if(in_array((int)$record['properties_values_id'], $selectedPropertyValues)) {
                $result['properties'][$record['properties_id']]['values'][(int)$record['properties_values_id']]['selected'] = true;
                $result['properties'][$record['properties_id']]['selected'] = true;
                $indexedSelected[(int)$record['properties_id']] = (int)(int)$record['properties_values_id'];
            }
        }
        $result['selected_values'] = $indexedSelected;
        return $result;
    }
    
    
    /**
     * @inheritDoc
     * @throws DBALException
     */
    public function getModifierBySellingUnit(SellingUnitId $id, LanguageId $languageId): ModifierDTOCollectionInterface
    {
        $result    = new ModifierDTOCollection();
        try {
            $configuration = $this->getProductPropertiesConfiguration($id);
        } catch (DBALException $e) {
            $configuration = [];
        }
        if(!count($configuration) || !count($configuration['properties']))
            return $result;

        $validValues = [];
        if($configuration['properties_dropdown_mode']) {
            $validValues = $this->getSelectablePropertyValues(
                $id->productId()->value(),
                $configuration
            );
        }
        
        foreach ($configuration['properties'] as $property_id => $property) {
            foreach ($property['values'] as $properties_values_id => $value) {
        
        
                [$pricePrefix, $priceValueFloat] = $this->pricePrefixAndValueFromFloat((float)$value['value_price']);
        
                $propertyId = new PropertyModifierIdentifier($properties_values_id);
        
                $result->addModifier($this->builder->withId($propertyId)
                                         ->withGroupId(new PropertyGroupIdentifier($property_id))
                                         ->withType($property['display_type'])
                                         ->withName($value['values_name'])
                                         ->withSelectable($value['selectable'] || in_array($properties_values_id, $validValues))
                                         ->withSelected($value['selected'])
                                         ->withPrice($priceValueFloat)
                                         ->withPricePrefix($pricePrefix)
                                         ->withImage($value['display_image'] ? 'product_images/property_images/'
                                                                              . $value['display_image'] : '')
                                         ->withSource('property')
                                         ->build());
            }
        }

        return $result;
    }

    
    
    /**
     * @param int $configuration
     *
     * @return string
     */
    public function stockField(int $configuration) : ?string {
        switch ($configuration) {
            case 0:
                return $this->attributeStockCheck ? '`ppc`.`combi_quantity`' : '`products`.`products_quantity`';
            case 1:
                return '`products`.`products_quantity`';
            case 2:
                return '`ppc`.`combi_quantity`';
            default:
                return '';
        }
    }
    
    protected function stockCheck(int $configuration): bool
    {
        /**
         * 0 = Use General configuration
         * 1 = Use product Quantity
         * 2 = Use combination configuration
         * 3 = Dont check stock
         */
        switch ($configuration) {
            case 0:
            case 1:
            case 2:
                return $this->stockCheck && $this->checkStockBeforeShoppingCart && !$this->stockAllowCheckout;
            default:
                return false;
        }
    }
    
    
    /**
     * @param int   $productId
     * @param array $configuration
     *
     * @return array
     * @throws DBALException
     */
    protected function getSelectablePropertyValues(int $productId, array $configuration): array
    {
        $specificOrder = $configuration['properties_dropdown_mode'] == 'dropdown_mode_2';
        $previousWasSelected = $specificOrder;
        $fields = '';
        $joins = '';
        $whereFieldsArray = [];
        $sep = "";
        foreach ($configuration['properties'] as $properties_id => $property) {
            if(($specificOrder && $previousWasSelected) || !$specificOrder) {
                $fields .= "\n {$sep}ppcv{$properties_id}.`properties_values_id` `{$properties_id}`";
                $joins  .= "\n inner join products_properties_combis_values `ppcv{$properties_id}`  on ppc.products_properties_combis_id = ppcv{$properties_id}.products_properties_combis_id";
                $joins  .= "\n inner join  `properties_values` `p{$properties_id}` on `p{$properties_id}`.`properties_values_id` = `ppcv{$properties_id}`.`properties_values_id` and `p{$properties_id}`.`properties_id` = {$properties_id}";
                $previousWasSelected = $property['selected'];
                if ($property['selected']) {
                    $whereFieldsArray[] = "`p{$properties_id}`.`properties_values_id`";
                }
                $sep = ",";
            }
        }
        
        $sql = <<<SQL
            SELECT distinct
                $fields
            FROM products
            inner join  products_properties_combis ppc on products.products_id = ppc.products_id
            $joins
            where products.`products_id` = {$productId}
SQL;
        /* a combination to be valid must have a least n-1 matches with the selected properties*/
        if(count($configuration['selected_values']) > 1) {
            $values = implode(',',$configuration['selected_values']);
            foreach ($whereFieldsArray as $key=> $field) {
                $whereFieldsArray[$key] =  "if($field in ($values),1,0)";
            }
            $sql.=" and (".implode(' + ', $whereFieldsArray).') >= '.(count($whereFieldsArray) - 1);
        }

        if($this->stockCheck($configuration['use_properties_combis_quantity'])
           && $stockField = $this->stockField($configuration['use_properties_combis_quantity'])) {
            $sql.= "\nand {$stockField} >= products.gm_min_order";
        }
    
        $combinations = [];
        $data = $this->queryBuilder->query($sql)->result_array();
        foreach ($data as $row) {
            $combinations[] = array_map('intval', $row);
        }
    
        $selectablePropertyValues = [];
        foreach ($configuration['properties'] as $property_id => $property) {
            foreach ($combinations as $index => $combination) {
                if(isset($combination[$property_id])) {
                    $partialCombination  = $combination;
                    $partialSelection    = $configuration['selected_values'];
                    $selectedOptionValue = $partialCombination[$property_id];
                    unset($partialCombination[$property_id], $partialSelection[$property_id]);
                    if (count($partialCombination) != count($partialSelection)) {
                        foreach ($partialCombination as $propId => $valueId) {
                            if (!isset($partialSelection[$propId])) {
                                $partialSelection[$propId] = $valueId;
                            }
                        }
                    }
                    /**
                     * the user can change only one option every time,
                     * to allow this combination to be selected must have a perfect match between the other property values
                     * */
                    if (count(array_intersect($partialCombination, $partialSelection)) == count($partialSelection)) {
                        $selectablePropertyValues[$selectedOptionValue] = $selectedOptionValue;
                    }
                }
            }
        }

        return array_values($selectablePropertyValues);
    }
    
    protected function selectedPropertyValues(SellingUnitId $id)
    {
        $result = [];
        foreach ($id->modifiers() as $modifier){
            if($modifier instanceof PropertyModifierIdentifier)
                $result[] = $modifier->value();
            }
        return $result;
    }

}