<?php
/*------------------------------------------------------------------------------
 OnGetSellingUnitPriceEventListener.php 2020-10-28
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\Properties\SellingUnitPrice\Listener;

use Exception;
use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollectionInterface;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertiesReaderServiceInterface;
use Gambio\Shop\Properties\Properties\Entities\CheapestCombination;
use Gambio\Shop\SellingUnit\Database\Unit\Events\Interfaces\OnGetSellingUnitPriceEventInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\PriceFormatted;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\PricePlain;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\PriceStatus;
use PropertiesControlInterface;
use stdClass;

/**
 * Class OnGetSellingUnitPriceEventListener
 * @package Gambio\Shop\Properties\SellingUnitPrice\Listener
 */
class OnGetSellingUnitPriceEventListener
{
    /**
     * @var PropertiesReaderServiceInterface
     */
    protected $readService;
    
    /**
     * @var PropertiesControlInterface
     */
    protected $propertiesControl;

    
    
    /**
     * OnGetSellingUnitPriceEventListener constructor.
     *
     * @param PropertiesReaderServiceInterface        $readService
     * @param PropertiesControlInterface  $propertiesControl
     */
    public function __construct(
        PropertiesReaderServiceInterface $readService,
        PropertiesControlInterface $propertiesControl
    ) {
        $this->readService       = $readService;
        $this->propertiesControl = $propertiesControl;
    }
    
    
    /**
     * @param OnGetSellingUnitPriceEventInterface $event
     */
    public function __invoke(OnGetSellingUnitPriceEventInterface $event)
    {
        try {
            $cheapestCombination = $this->readService->getCheapestCombinationsFor($event->id());
            if ($cheapestCombination) {
                $event->builder()
                    ->withData('combination', $cheapestCombination->id()->value())
                    ->withData('cheapest', $cheapestCombination instanceof CheapestCombination);
                
            }
        } catch (Exception $e){
            //silent the exception
        }
    }
    
    
    /**
     * @param array $propertyValuesIds
     *
     * @return bool
     */
    protected function allPropertyValueIdsAreNumeric(array $propertyValuesIds): bool
    {
        if (empty($propertyValuesIds)) {
            
            return false;
        }
        
        $isNumericResultArray = array_map(static function (int $propertyValueId) {
            return is_numeric($propertyValueId);
        },
            $propertyValuesIds);
        
        return in_array(false, $isNumericResultArray, true) === false;
    }
    
    
    /**
     * @param ModifierIdentifierCollectionInterface $modifiers
     *
     * @return array
     */
    protected function propertiesValuesIds(ModifierIdentifierCollectionInterface $modifiers): array
    {
        $propertyValuesIds = [];
    
        foreach ($modifiers as $modifier) {
            
            if ($modifier->type() === 'property') {
            
                $propertyValuesIds[] = $modifier->value();
            }
        }
        
        return $propertyValuesIds;
    }
    
    
    /**
     * @param float     $quantity
     * @param array     $propertiesValuesIds
     * @param ProductId $productId
     *
     * @return mixed
     */
    protected function getPropertiesSelectionTemplate(float $quantity, array $propertiesValuesIds, ProductId $productId)
    {
        $propertiesValuesArray = array();

        foreach($propertiesValuesIds as $key => $property_value_id)
        {
            $propertiesValuesArray[] = $key . ':' . $property_value_id;
        }

        $propertiesValues = implode('&', $propertiesValuesArray);

        return $this->propertiesControl->get_selection_data($productId->value(),
            $_SESSION['languages_id'],
            $quantity,
            $propertiesValues,
            $_SESSION['currency'],
            $_SESSION['customers_status']['customers_status_id'],
            false,
            false);
    }
    
    /**
     * @param float $pricePlain
     *
     * @return PricePlain
     */
    protected function pricePlain(float $pricePlain): PricePlain
    {
        return new PricePlain($pricePlain);
    }
    
    
    /**
     * @param string $priceFormatted
     *
     * @return PriceFormatted
     */
    protected function priceFormatted(string $priceFormatted): PriceFormatted
    {
        return new PriceFormatted($priceFormatted);
    }

    /**
     * @param int $priceStatus
     * @return PriceStatus
     */
    protected function priceStatus(int $priceStatus) : PriceStatus
    {
        return new PriceStatus($priceStatus);

    }
}